tic %should take around 20 seconds
force_file_name = 'force.xvg';
coord_file_name = 'coord.xvg';

nsteps = 100001;
dt = 0.010; %ps

N_A = 6.022e23;
c = 2.9979e10;
h = 6.626e-34;
mass = 1.66e-27;
Delta = 240; %
w_0 = 3550;
%for mass weighted forces
mH = 1;
mO = 16;
red_mass = mH*mO/(mH+mO);

a = -sqrt(9/4*(Delta/w_0)*1e18*1e6/N_A^2 / (w_0*c*h) / (red_mass*mass)*1e-24)/(2*pi*c*1e-12);

force_fid = fopen(force_file_name);
coord_fid = fopen(coord_file_name);

posO    = zeros(3,1);
posH1   = zeros(3,1);
forceO  = zeros(3,1);
forceH1 = zeros(3,1);

freq  = zeros(nsteps,1);

for i = 1:nsteps
    
    t = fscanf(coord_fid,'%f',1);
    posO  = fscanf(coord_fid,'%f',3);
    posH1 = fscanf(coord_fid,'%f',3);
    %posH2 = fscanf(coord_fid,'%f',3);
    
    t = fscanf(force_fid,'%f',1);
    forceO  = fscanf(force_fid,'%f',3);
    forceH1 = fscanf(force_fid,'%f',3);
    %forceH2 = fscanf(force_fid,'%f',3);
    
    bond = posH1 - posO;
    bond_length = sqrt(bond'*bond);
    u_bond = bond./bond_length;
        
    force = (forceH1/mH - forceO/mO)*red_mass;
    
    %projected on the OH bond vector
    force_proj = force'*u_bond;
    
    %convert field to frequency w_01
    freq(i) = a.*force_proj;
                
end

fclose(force_fid);
fclose(coord_fid);

freq = freq + w_0;

%calculate the deviation of the frequency from the mean and convert units
%to ps-1
dw = (freq - mean(freq))*(2*pi*c*1e-12); %in rad/ps-1
meanw = mean(freq)*(2*pi*c*1e-12); %in rad/ps-1
t = (0:nsteps-1)*dt;

figure(1),plot(t,freq);

%plot the frequency distribution
figure(2),hist(freq,100);

%% Calculate the frequency correlation function
n = length(dw);
ncorr = 150;
nskip = 5;
corr = zeros(1,ncorr);
tcorr = (0:ncorr-1)*dt;
for i = 0:ncorr-1
    for iti = 0:5:n-ncorr-1
        corr(i+1) = corr(i+1) + dw(iti+1)*dw(iti + i + 1);
    end
end
corr = corr/(n-ncorr)*nskip;
%normalize to unity
D = sqrt(corr(1));
corr = corr/D^2;

figure(3),plot(tcorr,D^2.*corr)

%% fit the FFCF 

%the functional form as a sum of exponentials and one damped cosine
fit = @(p,t) (p(1).*cos(p(2).*t).*exp(-p(3).*t)+p(4).*exp(-p(5).*t)+p(6).*exp(-p(7).*t));

%the function to minimize is the sum of squares of the difference between
%the data <corr> and the fitting function
fxn = @(p,corr,t) sum((corr - fit(p,t)).^2);

%initital paramters
a1 = 0.3232;
k11 = 30.01; %ps-1
k12 = 17.41; %ps-1
a2 = 0.3378;
k2 = 8.270; %ps-1
a3 = 0.3455;
k3 = 1.897; %ps-1

p0 = [a1 k11 k12 a2 k2 a3 k3];
figure(4),plot(tcorr,corr,tcorr,fit(p0,tcorr));
title('initial guess');

% do the fit
p = fminsearch(@(p) fxn(p,corr,tcorr),p0);

%plot the results
figure(5),plot(tcorr,corr,tcorr,fit(p,tcorr));
title('fit results');

%%
  
%unpack the results
a1 = p(1);
k11 = p(2);
k12 = p(3);
a2 = p(4);
k2 = p(5);
a3 = p(6);
k3 = p(7);

%the analytical result of the double integration is our new lineshape
%function
g = @(t) D^2*exp(-t.*(k12+k2+k3))/(k2^2*k3^2*(k11^2+k12^2)^2) ...
    .*(a1*k2^2*k3^2.*exp((k2+k3).*t).*(cos(k11.*t).*(k12^2-k11^2)-2*k11*k12*sin(k11*t) ...
    + exp(k12.*t).*(k11^2*(k12.*t+1)+k12^2*(k12.*t-1))) ...
    + (k11^2+k12^2)^2.*exp(k12.*t).*(a3*k2^2*exp(k2.*t).*(exp(k3.*t).*(k3.*t-1)+1) ...
    +a2*k3^2*exp(k3.*t).*(exp(k2.*t).*(k2*t-1)+1)));

%%
Delta_ps = Delta*(2*pi*c*1e-12);

nt = 512;
dtt = 0.0025;
tt = (0:nt-1)*dtt;
[T1,T3] = meshgrid(tt,tt);
t2 = 0.5;

% change this flag to see the effect of the overlap of v=0->1 and v=1->2
% transitions
flag_twolevelsystem = 0; 
if flag_twolevelsystem
    %only 0-1 transition
    R_nr = exp(-g(T1)-g(t2)-g(T3)+g(T1+t2)+g(t2+T3)-g(T1+t2+T3))...
        .*(exp(-1i.*(T3+T1)*meanw));
    R_r  = exp(-g(T1)+g(t2)-g(T3)-g(T1+t2)-g(t2+T3)+g(T1+t2+T3))...
        .*(exp(-1i.*(T3-T1)*meanw));
else
    %add excited state absorption shifted by the anharmonicity Delta in
    %units of radians/ps
    R_nr = exp(-g(T1)-g(t2)-g(T3)+g(T1+t2)+g(t2+T3)-g(T1+t2+T3))...
        .*(exp(-1i.*(T3+T1)*meanw) - exp(-1i.*(T3.*(meanw-Delta_ps)+T1*meanw)));
    R_r  = exp(-g(T1)+g(t2)-g(T3)-g(T1+t2)-g(t2+T3)+g(T1+t2+T3))...
        .*(exp(-1i.*(T3-T1)*meanw) - exp(-1i.*(T3.*(meanw-Delta_ps)-T1*meanw)));
end

%%
figure(7)
contourf(real(R_r),4)
figure(8)
contourf(real(R_nr),4)

%% divide the first points by 2 to suppress the baseline artifact
R_r(:,1)  = R_r(:,1)./2;
R_r(1,:)  = R_r(1,:)./2;
R_nr(:,1) = R_nr(:,1)./2;
R_nr(1,:) = R_nr(1,:)./2;


%do the fft. Note due to the definition of frequencies used above (ie
%exp(-iwt) we use the ifft to extract positive frequencies
R_nr_f = fftshift(ifft2(R_nr,2*nt,2*nt));
R_r_f  = fftshift(ifft2(R_r, 2*nt,2*nt));
R_r_f  = fliplr(  circshift(R_r_f,[0 -1]));
R    = real(R_r_f + R_nr_f);

%calculate the frequency axis
a = 1/dtt/(c*1e-12);
dw1 = a/(2*length(tt));
w1 = (-a/2+dw1:dw1:a/2);
w3 = w1;

%some things for plotting
n_contours = 20;
range1 = [3100 3600];
ind1 = find(w1>=range1(1) & w1<=range1(2));
range3 = [2900 3600];
ind3 = find(w1>=range3(1) & w1<=range3(2));

figure(9)
contourf(w3(ind3),w1(ind1),R(ind3,ind1)',n_contours)
line([range1(1) range1(end)], [range1(1) range1(end)],'Color','k');
axis equal tight

toc
    