tic %whole script should take about 30 seconds
force_file_name = 'force.xvg';
coord_file_name = 'coord.xvg';

nsteps = 100001;
dt = 0.010; %fs

N_A = 6.022e23;
c = 2.9979e10;
h = 6.626e-34;
mass = 1.66e-27;
Delta = 240; %
w_0 = 3550;
%for mass weighted forces
mH = 1;
mO = 16;
red_mass = mH*mO/(mH+mO);

a = -sqrt(9/4*(Delta/w_0)*1e18*1e6/N_A^2 / (w_0*c*h) / (red_mass*mass)*1e-24)/(2*pi*c*1e-12);

force_fid = fopen(force_file_name);
coord_fid = fopen(coord_file_name);

posO    = zeros(3,1);
posH1   = zeros(3,1);
forceO  = zeros(3,1);
forceH1 = zeros(3,1);

freq  = zeros(nsteps,1);

for i = 1:nsteps
    
    t = fscanf(coord_fid,'%f',1);
    posO  = fscanf(coord_fid,'%f',3);
    posH1 = fscanf(coord_fid,'%f',3);
    %posH2 = fscanf(coord_fid,'%f',3);
    
    t = fscanf(force_fid,'%f',1);
    forceO  = fscanf(force_fid,'%f',3);
    forceH1 = fscanf(force_fid,'%f',3);
    %forceH2 = fscanf(force_fid,'%f',3);
    
    bond = posH1 - posO;
    bond_length = sqrt(bond'*bond);
    u_bond = bond./bond_length;
        
    force = (forceH1/mH - forceO/mO)*red_mass;
    
    %projected on the OH bond vector
    force_proj = force'*u_bond;
    
    %convert field to frequency w_01
    freq(i) = a.*force_proj;
                
end

fclose(force_fid);
fclose(coord_fid);

dw = freq - mean(freq);
t = (0:nsteps-1)*dt;

figure(1),plot(t,freq);
figure(2),hist(freq,100);
figure(3),plot(t,dw);

%% 1D spectra
nt = 20;
nskip = 1;
ntint = 4;
ntrajectories = floor((nsteps/ntint - nt)/nskip);
mean_w = mean(freq);
dt = 0.01*ntint;

%convert from wavenumbers to rad/ps-1
dw = (freq-mean_w)*2*pi*c*1e-12;

%take every 4th point
%dw = dw(1:ntint:end);

%take an average over ntint points
dw = dw(1:floor(nsteps/ntint)*ntint);
dw = trapz(reshape(dw,ntint,[]),1)./(ntint-1);

%1D spectrum
S  = zeros(1,nt);
Sf = zeros(1,2*nt);

%build a matrix DW1 from the frequency trajectory consisting of each block of
%that trajectory that we will integrate and average. Each column has nt
%points with a different time origin given by <offset>
DW1 = zeros(nt,ntrajectories);
offset = 1:nskip:nsteps/ntint-nt;
for is = 1:ntrajectories
    DW1(:,is) = dw(offset(is):offset(is)+nt-1);
end

%calculate the time integral for each minitrajectory
sdw = cumtrapz(DW1,1)*dt;

%then calcuate the ensemble average of exp(1i <time integral>)
S = sum(exp(-1i*sdw),2);

%normalize
S = S*nskip/(nsteps/ntint-nt);

%the first time points need to be divided by 2 (Sect 9.5.3)
S(1) = S(1)/2;

%do the fft
Sf = real(fftshift(fft(S,2*nt)));

%plot
t1 = (0:nt-1)*dt;
t3 = t1;

figure(4),
plot((0:nt-1)*ntint,dw(1:nt),'o',...
    (0:nt-1)*ntint,DW1(:,1),'s',...
    0:(nt-1)*ntint,(freq(1:(nt-1)*ntint+1)-mean_w)*2*pi*c*1e-12);


figure(5)
plot(Sf,'-o')

%% 2D spectrum
it2 = 0;
ntrajectories = floor((nsteps/ntint - 2*nt + it2)/nskip);
R_r  = zeros(nt,nt);
R_nr = zeros(nt,nt);
R = zeros(2*nt,2*nt);

%build a matrix DW1 from the frequency trajectory consisting of each block of
%that trajectory that we will integrate and average. Each column has nt
%points with a different time origin given by <offset>. For DW2 we do the
%same but as a function of t1 and t3
DW1 = zeros(nt,ntrajectories);
DW2 = zeros(nt,nt,ntrajectories);

offset1 = 1:nskip:nsteps/ntint-nt;
for is = 1:ntrajectories
    DW1(:,is) = dw(offset1(is):offset1(is)+nt-1);
end
for is = 1:ntrajectories
    for it1 = 1:nt
        DW2(it1,:,is) = dw(offset1(is)+it1+it2:offset1(is)+it1+it2+nt-1);
    end
end

%calculate the time integral for each minitrajectory
% sdw has size (nt,trajectories). The cumulative trapz results in each of
% the nt elements representing the integral of the DW1 with an increasing
% upper limit of integration 
sdw = cumtrapz(DW1,1)*dt;

%copy this out into a matrix for (t1,t3)
SDW = repmat(reshape(sdw,[nt 1 ntrajectories]),[1 nt 1]);

%for the t3 dependence
PDW = cumtrapz(DW2,2)*dt;

R_r  = sum(exp(-1i.*PDW + 1i*SDW),3);
R_nr = sum(exp(-1i.*PDW - 1i*SDW),3);

%normalize
R_r  = R_r *nskip/(nsteps/ntint-nt);
R_nr = R_nr*nskip/(nsteps/ntint-nt);

figure(7)
contourf(real(R_r))
figure(8)
contourf(real(R_nr))

%%
%add the excited state absorption
[T1,T3] = meshgrid(t1,t3);

R_r_anh = (2*exp(1i*Delta/2*(2*pi*c*1e-12).*T3) - 2*exp(-1i*Delta/2/(2*pi*c*1e-12).*T3)).*R_r;
R_nr_anh = (2*exp(1i*Delta/2*(2*pi*c*1e-12).*T3) - 2*exp(-1i*Delta/2/(2*pi*c*1e-12).*T3)).*R_nr;

%the first time points need to be divided by 2 (Sect 9.5.3)
R_r(:,1) = R_r(:,1)/2;
R_r(1,:) = R_r(1,:)/2;
R_nr(:,1) = R_nr(:,1)/2;
R_nr(1,:) = R_nr(1,:)/2;

%do the fft
R_nr_f = fftshift(fft2(R_nr,2*nt,2*nt));
R_r_f  = fftshift(fft2(R_r, 2*nt,2*nt));
R_r_f  = fliplr(  circshift(R_r_f,[0 -1]));
R    = real(R_r_f + R_nr_f);

%the first time points need to be divided by 2 (Sect 9.5.3)
R_r_anh(:,1) = R_r_anh(:,1)/2;
R_r_anh(1,:) = R_r_anh(1,:)/2;
R_nr_anh(:,1) = R_nr_anh(:,1)/2;
R_nr_anh(1,:) = R_nr_anh(1,:)/2;

%do the fft
R_nr_anh_f = fftshift(fft2(R_nr_anh,2*nt,2*nt));
R_r_anh_f  = fftshift(fft2(R_r_anh, 2*nt,2*nt));
R_r_anh_f = fliplr(circshift(R_r_anh_f,[0 -1]));
R_anh    = real(R_r_anh_f + R_nr_anh_f);

figure(9)
contourf(R',10)


figure(10)
contourf(R_anh',10)

%%
toc
disp('done')

    